/***************************************************************************
 *   Copyright (C) 2004 by Thomas Lbking                                  *
 *   thomas@home                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "bab.h"
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlayout.h> 
#include <qtooltip.h>
#include <qvbox.h> 
#include <qpainter.h>
#include <dcopclient.h>
#include <kapplication.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>

static const char description[] =
    I18N_NOOP("Baghira Aqua/Brushed");

static const char version[] = "0.1";

static KCmdLineOptions options[] =
{
//    { "+[URL]", I18N_NOOP( "Document to open." ), 0 },
    KCmdLineLastOption
};

int main(int argc, char **argv)
{
    KAboutData about("bab", I18N_NOOP("BAB - Baghira Aqua/Brushed Switcher"), version, description,
                     KAboutData::License_GPL, "(C) 2004 Thomas Lübking", 0, 0, "thomas@home");
    about.addAuthor( "Thomas Lübking", 0, "thomas@home" );
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions( options );
    KApplication app;
    DCOPClient* client = app.dcopClient();
    client->registerAs( "bab", false);
    client->setDefaultObject( "babInterface" );
//    BabSwitcher* switcher = 0;
    bab* mainBAB = 0;

     if (app.isRestored())
     {
         RESTORE(bab);
	 mainBAB = new bab;
         app.setMainWidget(mainBAB);
//          mainBAB->show();
//        switcher->setObjId("bab");
//        switcher->show();
     }
     else
     {   
        // no session.. just start up normally
        KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

        /// @todo do something with the command line args here

//        switcher = new BabSwitcher;
        
        mainBAB = new bab;
        app.setMainWidget(mainBAB);
//          mainBAB->show();
//        switcher->setObjId("bab");
//        switcher->show();
        
        args->clear();
    }

    int ret = app.exec();
//    if (switcher) delete switcher;
    if (mainBAB) delete mainBAB;
    return ret;
}


bab::bab() : KMainWindow(0,"bab",Qt::WStyle_StaysOnTop | Qt::WX11BypassWM ){
   bSwitcher = new BabSwitcher(this,"switcher");
   QToolTip::add(bSwitcher, i18n("leftclick toggles state<br>rightclick toggles setupmenu"));
   bSwitcher->show();
    
   QSettings settings;  
   QVBox *vb = new QVBox(this);
//    QVBoxLayout *vbl = new QVBoxLayout(vb);
   vb->setLineWidth(2);
   vb->setMargin(2);
   vb->setFrameStyle(QFrame::Panel | QFrame::Plain );   
   QLabel *DefStyleLabel = new QLabel("Default Style is", vb);
   DefStyle = new QComboBox(vb);
   DefStyle->insertItem ( "Aqua" );
   DefStyle->insertItem ( "Brushed Metal" );
   DefStyle->setCurrentItem(settings.readNumEntry("/qt/BAB/defaultState", BabSwitcher::Aqua ));
   QLabel *AquaTabLabel = new QLabel("Aqua Tabs are", vb);
   AquaTab = new QComboBox(vb);
   AquaTab->insertItem ( "Tab" );
   AquaTab->insertItem ( "Clever" );
   AquaTab->insertItem ( "Chooser" );
   AquaTab->setCurrentItem(settings.readNumEntry("/qt/BAB/aquaTab", Clever ));
   QLabel *BrushedTabLabel = new QLabel("Brushed Tabs are", vb);
   BrushedTab = new QComboBox(vb);
   BrushedTab->insertItem ( "Tab" );
   BrushedTab->insertItem ( "Clever" );
   BrushedTab->insertItem ( "Chooser" );
   BrushedTab->setCurrentItem(settings.readNumEntry("/qt/BAB/brushedTab", Clever ));
   QPushButton *Quit = new QPushButton("Quit BAB", vb);
//   Quit->setPixmap(BarIcon("fileclose"));
   QPushButton *Close = new QPushButton("Close Menu", vb);
   vb->adjustSize();
   
   bSwitcher->setState(DefStyle->currentItem());
   
   connect(DefStyle, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(AquaTab, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(BrushedTab, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(AquaTab, SIGNAL(activated(int)), bSwitcher, SLOT(saveState()));
   connect(BrushedTab, SIGNAL(activated(int)), bSwitcher, SLOT(saveState()));
   connect(Close, SIGNAL(clicked()), this, SLOT(hide()));
   connect(Quit, SIGNAL(clicked()), this, SLOT(close()));
   
   resize(vb->width(),vb->height());
   
//   show();
}

bab::~ bab(){
   bSwitcher->setState(DefStyle->currentItem());
   if (bSwitcher) delete bSwitcher;
}

/** overwrihgt the show event for repositioning the window on top
 of the icon, adapted from kkeyled */
void bab::showEvent(QShowEvent *e){
    if ( bSwitcher->globalPos->x() > 0) {
       QWidget *desktop = QApplication::desktop();
       int w = desktop->width();
       QPoint nPos;   
       if(bSwitcher->globalPos->y() - height() > 0 ) { // it's not on top
          nPos.setY(bSwitcher->globalPos->y() -height());
          nPos.setX(bSwitcher->globalPos->x() - (width() / 2));
       } else { 
          if( bSwitcher->globalPos->y() > 0 ) { 
               nPos.setY(bSwitcher->globalPos->y() + bSwitcher->height() );
               nPos.setX(bSwitcher->globalPos->x() - (width() / 2));
          }
       }
       if ( nPos.x() < 0 ) {
          nPos.setX(bSwitcher->globalPos->x() + bSwitcher->width());
       } else {
          if ( bSwitcher->globalPos->x() + (width() / 2)  > w ) {
               nPos.setX(bSwitcher->globalPos->x() - width());
          }
       }
       move(nPos);
    }
    
   // If it was set by the normal sessionmanagement it needs
   // to set the Flags again
   setWFlags(Qt::WStyle_StaysOnTop | Qt::WX11BypassWM); 
   QWidget::showEvent(e); // execute the normal showevent
   raise();
}

void bab::saveSettings(){
   QSettings * settings = new QSettings;
   settings->writeEntry("/qt/BAB/defaultState", DefStyle->currentItem());
   settings->writeEntry("/qt/BAB/aquaTab", AquaTab->currentItem());
   settings->writeEntry("/qt/BAB/brushedTab", BrushedTab->currentItem() );
   delete settings;
}

QPoint *BabSwitcher::globalPos = new QPoint::QPoint(0,0);

BabSwitcher::BabSwitcher(bab *parent, const char *name) 
                                          : KSystemTray(parent,name), DCOPObject("babInterface") {
   KIconLoader* iLoader = KGlobal::iconLoader();
   aquaIcon = iLoader->loadIcon("bab-aqua", KIcon::Small, 22);
   brushedIcon = iLoader->loadIcon("bab-brushed", KIcon::Small, 22);
   readState();
   installEventFilter(this);                                          
}

BabSwitcher::~BabSwitcher(){
}

void BabSwitcher::toggleState(){
   if ( state_ == Aqua ) {
      // lade brushed icon...
      state_ = Brushed;
   }
   else {
      // lade aqua icon
      state_ = Aqua;
   }
   saveState();
   repaint();
}

void BabSwitcher::setState(int state){
   state_ = state < 0 ? (State)0 : state > 1 ? (State)1 : (State)state;
   saveState();
   repaint();
}

void BabSwitcher::setDecoState(int state){
   decoState_ = state < 0 ? 0 : state > 2 ? 2 : state;
   saveDecoState();
}

int BabSwitcher::state(){
   return (int)state_;
}

int BabSwitcher::decoState(){
   return decoState_;
}

void BabSwitcher::saveState(){
   QSettings *settings = new QSettings;
   settings->writeEntry("/qt/Baghira/brushMe", (state_ == Brushed) );
   settings->writeEntry("/qt/Baghira/TabStyle", 
                        (state_ == Aqua) ?
                        ((bab*)parent())->AquaTab->currentItem() :
                        ((bab*)parent())->BrushedTab->currentItem() );
   delete settings;
}

void BabSwitcher::saveDecoState(){
   KConfig *config = new KConfig("kwinbaghirarc");
   config->setGroup("General");
   config->writeEntry("defaultMode", decoState_);
   delete config;
}

void BabSwitcher::readState(){
   QSettings settings;
   state_ = (settings.readBoolEntry("/qt/Baghira/brushMe", false ) ? Brushed : Aqua);
   KConfig config("kwinbaghirarc");
   config.setGroup("General");
   decoState_ = config.readNumEntry("defaultMode",1);
}

void BabSwitcher::mousePressEvent(QMouseEvent *e){

  QWidget *daddy = parentWidget();

  globalPos->setX(e->globalX() - e->x());
  globalPos->setY(e->globalY() - e->y());

   switch ( e->button() ) {
   case LeftButton:{
      if ( state_ == Aqua ) {
      // lade brushed icon...
      state_ = Brushed;
      }
      else {
      // lade aqua icon
      state_ = Aqua;
      }
      saveState();
      repaint();
      break;
   }
   case RightButton:
   default:{
   // toggleMinimizeRestore();
      if ( daddy->isVisible() ) {
         daddy->hide();
      }
      else {
         daddy->show();
      }
   }
   }
}

void BabSwitcher::paintEvent(QPaintEvent *ev){
   KSystemTray::paintEvent(ev);
   QPainter *icyApainter = new QPainter(this);
   if(state_ == Aqua) {
      aquaIcon.isNull() ? icyApainter->fillRect(this->rect(), Qt::white) : icyApainter->drawPixmap(0, 0, aquaIcon);
   }
   else {
      brushedIcon.isNull() ? icyApainter->fillRect(this->rect(), Qt::black) : icyApainter->drawPixmap(0, 0, brushedIcon);
   }
   
   icyApainter->flush();
   icyApainter->end();
   delete icyApainter;
}

#include "bab.moc"
