//////////////////////////////////////////////////////////////////////////////
// baghiraconfig.cc
// -------------------
// Config module for Baghira window decoration
// -------------------
// Copyright (c) 2003, 2004 David Johnson <david@usermode.org>
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////

#include <kconfig.h>
#include <klocale.h>
#include <kglobal.h>
#include <kcolorbutton.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qwhatsthis.h>

#include "baghiraconfig.h"
#include "configdialog.h"
//#include "../images.h"

//////////////////////////////////////////////////////////////////////////////
// BaghiraConfig Class                                                        //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// BaghiraConfig()
// -------------
// Constructor

BaghiraConfig::BaghiraConfig(KConfig*, QWidget* parent)
    : QObject(parent), config_(0), dialog_(0)
{
    config_ = new KConfig("kwinbaghirarc");
    KGlobal::locale()->insertCatalogue("kwin_baghira_config");

    dialog_ = new ConfigDialog(parent);


    load(config_);
    dialog_->show();

    // connections
    connect(dialog_->titlealign, SIGNAL(clicked(int)),
            this, SLOT(selectionChanged()));
    connect(dialog_->drawComicFrame, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));            
    connect(dialog_->bigHandler, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->allowEasyClosing, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));            
    connect(dialog_->FollowStyle, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->defaultMode, SIGNAL(activated(int)), 
            this, SLOT(selectionChanged()));
    connect(dialog_->minTH, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));        
    
    connect(dialog_->activeColor1_J, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->activeColor2_J, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));            
    connect(dialog_->inactiveColor1_J, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->inactiveColor2_J, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->buttonStyle_J, SIGNAL(activated(int)),
    	    this, SLOT(selectionChanged()));
    connect(dialog_->shapeUL_J, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeUR_J, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLL_J, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLR_J, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->drawIcon_J, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));    
    connect(dialog_->titleeffect_J, SIGNAL(activated(int)), 
            this, SLOT(selectionChanged()));
    connect(dialog_->_3DImpact_J, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));
    connect(dialog_->LineImpact_J, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));          
            
    connect(dialog_->activeColor1_P, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->activeColor2_P, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));            
    connect(dialog_->inactiveColor1_P, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->inactiveColor2_P, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->buttonStyle_P, SIGNAL(activated(int)),
    	    this, SLOT(selectionChanged()));
    connect(dialog_->shapeUL_P, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeUR_P, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLL_P, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLR_P, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->drawIcon_P, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));    
    connect(dialog_->titleeffect_P, SIGNAL(activated(int)), 
            this, SLOT(selectionChanged()));
    connect(dialog_->_3DImpact_P, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));
    connect(dialog_->LineImpact_P, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));
            
    connect(dialog_->activeColor1_B, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->activeColor2_B, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));            
    connect(dialog_->inactiveColor1_B, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->inactiveColor2_B, SIGNAL(clicked()),
            this, SLOT(selectionChanged()));
    connect(dialog_->buttonStyle_B, SIGNAL(activated(int)),
    	    this, SLOT(selectionChanged()));
    connect(dialog_->shapeUL_B, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeUR_B, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLL_B, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLR_B, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->drawIcon_B, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));    
    connect(dialog_->titleeffect_B, SIGNAL(activated(int)), 
            this, SLOT(selectionChanged()));
    connect(dialog_->_3DImpact_B, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));
    connect(dialog_->LineImpact_B, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));

}

//////////////////////////////////////////////////////////////////////////////
// ~BaghiraConfig()
// --------------
// Destructor

BaghiraConfig::~BaghiraConfig()
{
    if (dialog_) delete dialog_;
    if (config_) delete config_;
}

//////////////////////////////////////////////////////////////////////////////
// selectionChanged()
// ------------------
// Selection has changed

void BaghiraConfig::selectionChanged()
{
    emit changed();
}

//////////////////////////////////////////////////////////////////////////////
// load()
// ------
// Load configuration data

void BaghiraConfig::load(KConfig*)
{
    config_->setGroup("General");
    
    QString value = config_->readEntry("TitleAlignment", "AlignHCenter");
    QRadioButton *button = (QRadioButton*)dialog_->titlealign->
        child((const char *)value.latin1());
    if (button) button->setChecked(true);

    dialog_->minTH->setValue(config_->readNumEntry("minimumTitleHeight",18));
    dialog_->bigHandler->setChecked(config_->readBoolEntry("BigHandler",false));
    dialog_->FollowStyle->setChecked(config_->readBoolEntry("FollowStyle",false));
    dialog_->allowEasyClosing->setChecked(config_->readBoolEntry("allowEasyClosing",false));
    dialog_->drawComicFrame->setChecked(config_->readBoolEntry("DrawComicFrame",false));
    dialog_->defaultMode->setCurrentItem(config_->readNumEntry("defaultMode",1));
        
    dialog_->activeColor1_J->setColor(QColor((unsigned int)config_->readNumEntry("activeColor1_1",QColor(250,250,250).rgb())));
    dialog_->inactiveColor1_J->setColor(QColor((unsigned int)config_->readNumEntry("inactiveColor1_1",QColor(216,231,247).rgb())));
    dialog_->activeColor2_J->setColor(QColor((unsigned int)config_->readNumEntry("activeColor2_1",QColor(220,220,220).rgb())));
    dialog_->inactiveColor2_J->setColor(QColor((unsigned int)config_->readNumEntry("inactiveColor2_1",QColor(212,220,235).rgb())));
    dialog_->buttonStyle_J->setCurrentItem(config_->readNumEntry("ButtonStyle_1",1));
    dialog_->titleeffect_J->setCurrentItem(config_->readNumEntry("TitleEffect_1", 1));
    dialog_->shapeUL_J->setChecked(config_->readBoolEntry("ShapeUL_1",true));
    dialog_->shapeUR_J->setChecked(config_->readBoolEntry("ShapeUR_1",true));
    dialog_->shapeLL_J->setChecked(config_->readBoolEntry("ShapeLL_1",false));
    dialog_->shapeLR_J->setChecked(config_->readBoolEntry("ShapeLR_1",false));
    dialog_->drawIcon_J->setChecked(config_->readBoolEntry("drawIcon_1",true));
    dialog_->_3DImpact_J->setValue(config_->readNumEntry("3DImpact_1",20));
    dialog_->LineImpact_J->setValue(config_->readNumEntry("LineImpact_1",40));
    
    dialog_->activeColor1_P->setColor(QColor((unsigned int)config_->readNumEntry("activeColor1_2",QColor(238,238,238).rgb())));
    dialog_->inactiveColor1_P->setColor(QColor((unsigned int)config_->readNumEntry("inactiveColor1_2",QColor(230,230,230).rgb())));
    dialog_->activeColor2_P->setColor(QColor((unsigned int)config_->readNumEntry("activeColor2_2",QColor(192,192,192).rgb())));
    dialog_->inactiveColor2_P->setColor(QColor((unsigned int)config_->readNumEntry("inactiveColor2_2",QColor(202,202,202).rgb())));
    dialog_->buttonStyle_P->setCurrentItem(config_->readNumEntry("ButtonStyle_2",0));
    dialog_->titleeffect_P->setCurrentItem(config_->readNumEntry("TitleEffect_2", 0));
    dialog_->shapeUL_P->setChecked(config_->readBoolEntry("ShapeUL_2",true));
    dialog_->shapeUR_P->setChecked(config_->readBoolEntry("ShapeUR_2",true));
    dialog_->shapeLL_P->setChecked(config_->readBoolEntry("ShapeLL_2",false));
    dialog_->shapeLR_P->setChecked(config_->readBoolEntry("ShapeLR_2",false));
    dialog_->drawIcon_P->setChecked(config_->readBoolEntry("drawIcon_2",true));
    dialog_->_3DImpact_P->setValue(config_->readNumEntry("3DImpact_2",20));
    dialog_->LineImpact_P->setValue(config_->readNumEntry("LineImpact_2",40));
    
    dialog_->activeColor1_B->setColor(QColor((unsigned int)config_->readNumEntry("activeColor1_3",QColor(238,238,238).rgb())));
    dialog_->inactiveColor1_B->setColor(QColor((unsigned int)config_->readNumEntry("inactiveColor1_3",QColor(230,230,230).rgb())));
    dialog_->activeColor2_B->setColor(QColor((unsigned int)config_->readNumEntry("activeColor2_3",QColor(192,192,192).rgb())));
    dialog_->inactiveColor2_B->setColor(QColor((unsigned int)config_->readNumEntry("inactiveColor2_3",QColor(202,202,202).rgb())));
    dialog_->buttonStyle_B->setCurrentItem(config_->readNumEntry("ButtonStyle_3",0));
    dialog_->titleeffect_B->setCurrentItem(config_->readNumEntry("TitleEffect_3", 4));
    dialog_->shapeUL_B->setChecked(config_->readBoolEntry("ShapeUL_3",true));
    dialog_->shapeUR_B->setChecked(config_->readBoolEntry("ShapeUR_3",true));
    dialog_->shapeLL_B->setChecked(config_->readBoolEntry("ShapeLL_3",true));
    dialog_->shapeLR_B->setChecked(config_->readBoolEntry("ShapeLR_3",true));
    dialog_->drawIcon_B->setChecked(config_->readBoolEntry("drawIcon_3",true));
    dialog_->_3DImpact_B->setValue(config_->readNumEntry("3DImpact_3",20));
    dialog_->LineImpact_B->setValue(config_->readNumEntry("LineImpact_3",40));

}

//////////////////////////////////////////////////////////////////////////////
// save()
// ------
// Save configuration data

void BaghiraConfig::save(KConfig*)
{
    config_->setGroup("General");

    QRadioButton *button = (QRadioButton*)dialog_->titlealign->selected();
    if (button) config_->writeEntry("TitleAlignment", QString(button->name()));
    config_->writeEntry("DrawComicFrame", dialog_->drawComicFrame->isChecked());
    config_->writeEntry("FollowStyle", dialog_->FollowStyle->isChecked());
    config_->writeEntry("BigHandler", dialog_->bigHandler->isChecked());
    config_->writeEntry("allowEasyClosing", dialog_->allowEasyClosing->isChecked());
    config_->writeEntry("defaultMode", dialog_->defaultMode->currentItem());
    config_->writeEntry("minimumTitleHeight", dialog_->minTH->value());
       
    config_->writeEntry("activeColor1_1", (int)dialog_->activeColor1_J->color().rgb());
    config_->writeEntry("activeColor2_1", (int)dialog_->activeColor2_J->color().rgb());
    config_->writeEntry("inactiveColor1_1", (int)dialog_->inactiveColor1_J->color().rgb());
    config_->writeEntry("inactiveColor2_1", (int)dialog_->inactiveColor2_J->color().rgb());
    config_->writeEntry("ButtonStyle_1", dialog_->buttonStyle_J->currentItem());
    config_->writeEntry("TitleEffect_1", dialog_->titleeffect_J->currentItem());
    config_->writeEntry("ShapeUL_1", dialog_->shapeUL_J->isChecked());
    config_->writeEntry("ShapeUR_1", dialog_->shapeUR_J->isChecked());
    config_->writeEntry("ShapeLL_1", dialog_->shapeLL_J->isChecked());
    config_->writeEntry("ShapeLR_1", dialog_->shapeLR_J->isChecked());    
    config_->writeEntry("drawIcon_1", dialog_->drawIcon_J->isChecked());
    config_->writeEntry("3DImpact_1", dialog_->_3DImpact_J->value());
    config_->writeEntry("LineImpact_1", dialog_->LineImpact_J->value());
    
    config_->writeEntry("activeColor1_2", (int)dialog_->activeColor1_P->color().rgb());
    config_->writeEntry("activeColor2_2", (int)dialog_->activeColor2_P->color().rgb());
    config_->writeEntry("inactiveColor1_2", (int)dialog_->inactiveColor1_P->color().rgb());
    config_->writeEntry("inactiveColor2_2", (int)dialog_->inactiveColor2_P->color().rgb());
    config_->writeEntry("ButtonStyle_2", dialog_->buttonStyle_P->currentItem());
    config_->writeEntry("TitleEffect_2", dialog_->titleeffect_P->currentItem());
    config_->writeEntry("ShapeUL_2", dialog_->shapeUL_P->isChecked());
    config_->writeEntry("ShapeUR_2", dialog_->shapeUR_P->isChecked());
    config_->writeEntry("ShapeLL_2", dialog_->shapeLL_P->isChecked());
    config_->writeEntry("ShapeLR_2", dialog_->shapeLR_P->isChecked());    
    config_->writeEntry("drawIcon_2", dialog_->drawIcon_P->isChecked());
    config_->writeEntry("3DImpact_2", dialog_->_3DImpact_P->value());
    config_->writeEntry("LineImpact_2", dialog_->LineImpact_P->value());
    
    config_->writeEntry("activeColor1_3", (int)dialog_->activeColor1_B->color().rgb());
    config_->writeEntry("activeColor2_3", (int)dialog_->activeColor2_B->color().rgb());
    config_->writeEntry("inactiveColor1_3", (int)dialog_->inactiveColor1_B->color().rgb());
    config_->writeEntry("inactiveColor2_3", (int)dialog_->inactiveColor2_B->color().rgb());
    config_->writeEntry("ButtonStyle_3", dialog_->buttonStyle_B->currentItem());
    config_->writeEntry("TitleEffect_3", dialog_->titleeffect_B->currentItem());
    config_->writeEntry("ShapeUL_3", dialog_->shapeUL_B->isChecked());
    config_->writeEntry("ShapeUR_3", dialog_->shapeUR_B->isChecked());
    config_->writeEntry("ShapeLL_3", dialog_->shapeLL_B->isChecked());
    config_->writeEntry("ShapeLR_3", dialog_->shapeLR_B->isChecked());    
    config_->writeEntry("drawIcon_3", dialog_->drawIcon_B->isChecked());
    config_->writeEntry("3DImpact_3", dialog_->_3DImpact_B->value());
    config_->writeEntry("LineImpact_3", dialog_->LineImpact_B->value());
    
    config_->sync();
}

//////////////////////////////////////////////////////////////////////////////
// defaults()
// ----------
// Set configuration defaults

void BaghiraConfig::defaults()
{
    QRadioButton *button = (QRadioButton*)dialog_->titlealign->child("AlignHCenter");
    if (button) button->setChecked(true);
    dialog_->bigHandler->setChecked(false);
    dialog_->allowEasyClosing->setChecked(false);
    dialog_->minTH->setValue(22);
    dialog_->FollowStyle->setChecked(true);
    dialog_->drawComicFrame->setChecked(false);
    dialog_->defaultMode->setCurrentItem(1);
    
   dialog_->activeColor1_J->setColor(QColor(250,250,250));
   dialog_->activeColor1_J->setColor(QColor(216,231,247));
   dialog_->activeColor1_J->setColor(QColor(220,220,220));
   dialog_->activeColor1_J->setColor(QColor(212,220,235));
   dialog_->buttonStyle_J->setCurrentItem(1);
   dialog_->titleeffect_J->setCurrentItem(1);
   dialog_->shapeUL_J->setChecked(true);
   dialog_->shapeUR_J->setChecked(true);
   dialog_->shapeLL_J->setChecked(false);
   dialog_->shapeLR_J->setChecked(false);
   dialog_->drawIcon_J->setChecked(true);
   dialog_->_3DImpact_J->setValue(20);
   dialog_->LineImpact_J->setValue(40);
   
   dialog_->activeColor1_P->setColor(QColor(238,238,238));
   dialog_->activeColor1_P->setColor(QColor(230,230,230));
   dialog_->activeColor1_P->setColor(QColor(192,192,192));
   dialog_->activeColor1_P->setColor(QColor(202,202,202));
   dialog_->buttonStyle_P->setCurrentItem(0);
   dialog_->titleeffect_P->setCurrentItem(0);
   dialog_->shapeUL_P->setChecked(true);
   dialog_->shapeUR_P->setChecked(true);
   dialog_->shapeLL_P->setChecked(false);
   dialog_->shapeLR_P->setChecked(false);
   dialog_->drawIcon_P->setChecked(true);
   dialog_->_3DImpact_P->setValue(20);
   dialog_->LineImpact_P->setValue(30);
   
   dialog_->activeColor1_B->setColor(QColor(238,238,238));
   dialog_->activeColor1_B->setColor(QColor(230,230,230));
   dialog_->activeColor1_B->setColor(QColor(192,192,192));
   dialog_->activeColor1_B->setColor(QColor(202,202,202));
   dialog_->buttonStyle_B->setCurrentItem(0);
   dialog_->titleeffect_B->setCurrentItem(4);
   dialog_->shapeUL_B->setChecked(true);
   dialog_->shapeUR_B->setChecked(true);
   dialog_->shapeLL_B->setChecked(false);
   dialog_->shapeLR_B->setChecked(false);
   dialog_->drawIcon_B->setChecked(true);
   dialog_->_3DImpact_B->setValue(20);
   dialog_->LineImpact_B->setValue(0);
    
}   
    
//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////
    
extern "C"
{   
    QObject* allocate_config(KConfig* config, QWidget* parent) {
        return(new BaghiraConfig(config, parent));
    }
}   
    
#include "baghiraconfig.moc"
